@@ -17,10 +17,11 @@
  * limitations under the License.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <winpr/cast.h>
 
+#include <freerdp/config.h>
+
+#include <freerdp/freerdp.h>
 #include <freerdp/channels/log.h>
 #include <freerdp/server/rdpecam.h>
 
@@ -75,10 +76,10 @@ static UINT device_server_open_channel(device_server* device)
 {
 	CameraDeviceServerContext* context = &device->context;
 	DWORD Error = ERROR_SUCCESS;
-	HANDLE hEvent;
+	HANDLE hEvent = NULL;
 	DWORD BytesReturned = 0;
 	PULONG pSessionId = NULL;
-	UINT32 channelId;
+	UINT32 channelId = 0;
 	BOOL status = TRUE;
 
 	WINPR_ASSERT(device);
@@ -122,7 +123,8 @@ static UINT device_server_open_channel(device_server* device)
 	return Error;
 }
 
-static UINT device_server_handle_success_response(CameraDeviceServerContext* context, wStream* s,
+static UINT device_server_handle_success_response(CameraDeviceServerContext* context,
+                                                  WINPR_ATTR_UNUSED wStream* s,
                                                   const CAM_SHARED_MSG_HEADER* header)
 {
 	CAM_SUCCESS_RESPONSE pdu = { 0 };
@@ -168,7 +170,6 @@ static UINT device_server_recv_stream_list_response(CameraDeviceServerContext* c
 {
 	CAM_STREAM_LIST_RESPONSE pdu = { 0 };
 	UINT error = CHANNEL_RC_OK;
-	BYTE i;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(header);
@@ -178,9 +179,12 @@ static UINT device_server_recv_stream_list_response(CameraDeviceServerContext* c
 	if (!Stream_CheckAndLogRequiredLength(TAG, s, 5))
 		return ERROR_NO_DATA;
 
-	pdu.N_Descriptions = MIN(Stream_GetRemainingLength(s) / 5, 255);
+	pdu.N_Descriptions = 255;
+	const size_t len = Stream_GetRemainingLength(s) / 5;
+	if (len < 255)
+		pdu.N_Descriptions = (BYTE)len;
 
-	for (i = 0; i < pdu.N_Descriptions; ++i)
+	for (BYTE i = 0; i < pdu.N_Descriptions; ++i)
 	{
 		CAM_STREAM_DESCRIPTION* StreamDescription = &pdu.StreamDescriptions[i];
 
@@ -203,7 +207,6 @@ static UINT device_server_recv_media_type_list_response(CameraDeviceServerContex
 {
 	CAM_MEDIA_TYPE_LIST_RESPONSE pdu = { 0 };
 	UINT error = CHANNEL_RC_OK;
-	BYTE i;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(header);
@@ -223,7 +226,7 @@ static UINT device_server_recv_media_type_list_response(CameraDeviceServerContex
 		return ERROR_NOT_ENOUGH_MEMORY;
 	}
 
-	for (i = 0; i < pdu.N_Descriptions; ++i)
+	for (size_t i = 0; i < pdu.N_Descriptions; ++i)
 	{
 		CAM_MEDIA_TYPE_DESCRIPTION* MediaTypeDescriptions = &pdu.MediaTypeDescriptions[i];
 
@@ -343,8 +346,6 @@ static UINT device_server_recv_property_list_response(CameraDeviceServerContext*
 
 	if (pdu.N_Properties > 0)
 	{
-		size_t i;
-
 		pdu.Properties = calloc(pdu.N_Properties, sizeof(CAM_PROPERTY_DESCRIPTION));
 		if (!pdu.Properties)
 		{
@@ -353,7 +354,7 @@ static UINT device_server_recv_property_list_response(CameraDeviceServerContext*
 			return ERROR_NOT_ENOUGH_MEMORY;
 		}
 
-		for (i = 0; i < pdu.N_Properties; ++i)
+		for (size_t i = 0; i < pdu.N_Properties; ++i)
 		{
 			Stream_Read_UINT8(s, pdu.Properties[i].PropertySet);
 			Stream_Read_UINT8(s, pdu.Properties[i].PropertyId);
@@ -401,11 +402,11 @@ static UINT device_server_recv_property_value_response(CameraDeviceServerContext
 
 static UINT device_process_message(device_server* device)
 {
-	BOOL rc;
+	BOOL rc = 0;
 	UINT error = ERROR_INTERNAL_ERROR;
-	ULONG BytesReturned;
+	ULONG BytesReturned = 0;
 	CAM_SHARED_MSG_HEADER header = { 0 };
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(device);
 	WINPR_ASSERT(device->device_channel);
@@ -431,7 +432,7 @@ static UINT device_process_message(device_server* device)
 		goto out;
 	}
 
-	if (WTSVirtualChannelRead(device->device_channel, 0, (PCHAR)Stream_Buffer(s),
+	if (WTSVirtualChannelRead(device->device_channel, 0, Stream_BufferAs(s, char),
 	                          (ULONG)Stream_Capacity(s), &BytesReturned) == FALSE)
 	{
 		WLog_ERR(TAG, "WTSVirtualChannelRead failed!");
@@ -506,6 +507,8 @@ static UINT device_server_context_poll_int(CameraDeviceServerContext* context)
 		case CAMERA_DEVICE_OPENED:
 			error = device_process_message(device);
 			break;
+		default:
+			break;
 	}
 
 	return error;
@@ -523,7 +526,7 @@ static HANDLE device_server_get_channel_handle(device_server* device)
 	                           &BytesReturned) == TRUE)
 	{
 		if (BytesReturned == sizeof(HANDLE))
-			CopyMemory(&ChannelEvent, buffer, sizeof(HANDLE));
+			ChannelEvent = *(HANDLE*)buffer;
 
 		WTSFreeMemory(buffer);
 	}
@@ -533,11 +536,11 @@ static HANDLE device_server_get_channel_handle(device_server* device)
 
 static DWORD WINAPI device_server_thread_func(LPVOID arg)
 {
-	DWORD nCount;
+	DWORD nCount = 0;
 	HANDLE events[2] = { 0 };
 	device_server* device = (device_server*)arg;
 	UINT error = CHANNEL_RC_OK;
-	DWORD status;
+	DWORD status = 0;
 
 	WINPR_ASSERT(device);
 
@@ -573,10 +576,12 @@ static DWORD WINAPI device_server_thread_func(LPVOID arg)
 						break;
 				}
 				break;
+			default:
+				break;
 		}
 	}
 
-	WTSVirtualChannelClose(device->device_channel);
+	(void)WTSVirtualChannelClose(device->device_channel);
 	device->device_channel = NULL;
 
 	if (error && device->context.rdpcontext)
@@ -606,7 +611,7 @@ static UINT device_server_open(CameraDeviceServerContext* context)
 		if (!device->thread)
 		{
 			WLog_ERR(TAG, "CreateThread failed!");
-			CloseHandle(device->stopEvent);
+			(void)CloseHandle(device->stopEvent);
 			device->stopEvent = NULL;
 			return ERROR_INTERNAL_ERROR;
 		}
@@ -625,7 +630,7 @@ static UINT device_server_close(CameraDeviceServerContext* context)
 
 	if (!device->externalThread && device->thread)
 	{
-		SetEvent(device->stopEvent);
+		(void)SetEvent(device->stopEvent);
 
 		if (WaitForSingleObject(device->thread, INFINITE) == WAIT_FAILED)
 		{
@@ -634,8 +639,8 @@ static UINT device_server_close(CameraDeviceServerContext* context)
 			return error;
 		}
 
-		CloseHandle(device->thread);
-		CloseHandle(device->stopEvent);
+		(void)CloseHandle(device->thread);
+		(void)CloseHandle(device->stopEvent);
 		device->thread = NULL;
 		device->stopEvent = NULL;
 	}
@@ -643,7 +648,7 @@ static UINT device_server_close(CameraDeviceServerContext* context)
 	{
 		if (device->state != CAMERA_DEVICE_INITIAL)
 		{
-			WTSVirtualChannelClose(device->device_channel);
+			(void)WTSVirtualChannelClose(device->device_channel);
 			device->device_channel = NULL;
 			device->state = CAMERA_DEVICE_INITIAL;
 		}
@@ -684,7 +689,7 @@ static BOOL device_server_context_handle(CameraDeviceServerContext* context, HAN
 
 static wStream* device_server_packet_new(size_t size, BYTE version, BYTE messageId)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	/* Allocate what we need plus header bytes */
 	s = Stream_New(NULL, size + CAM_HEADER_SIZE);
@@ -704,13 +709,15 @@ static UINT device_server_packet_send(CameraDeviceServerContext* context, wStrea
 {
 	device_server* device = (device_server*)context;
 	UINT error = CHANNEL_RC_OK;
-	ULONG written;
+	ULONG written = 0;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(s);
 
-	if (!WTSVirtualChannelWrite(device->device_channel, (PCHAR)Stream_Buffer(s),
-	                            Stream_GetPosition(s), &written))
+	const size_t len = Stream_GetPosition(s);
+	WINPR_ASSERT(len <= UINT32_MAX);
+	if (!WTSVirtualChannelWrite(device->device_channel, Stream_BufferAs(s, char), (UINT32)len,
+	                            &written))
 	{
 		WLog_ERR(TAG, "WTSVirtualChannelWrite failed!");
 		error = ERROR_INTERNAL_ERROR;
@@ -730,7 +737,7 @@ out:
 
 static UINT device_server_write_and_send_header(CameraDeviceServerContext* context, BYTE messageId)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 
@@ -741,9 +748,9 @@ static UINT device_server_write_and_send_header(CameraDeviceServerContext* conte
 	return device_server_packet_send(context, s);
 }
 
-static UINT
-device_send_activate_device_request_pdu(CameraDeviceServerContext* context,
-                                        const CAM_ACTIVATE_DEVICE_REQUEST* activateDeviceRequest)
+static UINT device_send_activate_device_request_pdu(
+    CameraDeviceServerContext* context,
+    WINPR_ATTR_UNUSED const CAM_ACTIVATE_DEVICE_REQUEST* activateDeviceRequest)
 {
 	WINPR_ASSERT(context);
 
@@ -752,15 +759,16 @@ device_send_activate_device_request_pdu(CameraDeviceServerContext* context,
 
 static UINT device_send_deactivate_device_request_pdu(
     CameraDeviceServerContext* context,
-    const CAM_DEACTIVATE_DEVICE_REQUEST* deactivateDeviceRequest)
+    WINPR_ATTR_UNUSED const CAM_DEACTIVATE_DEVICE_REQUEST* deactivateDeviceRequest)
 {
 	WINPR_ASSERT(context);
 
 	return device_server_write_and_send_header(context, CAM_MSG_ID_DeactivateDeviceRequest);
 }
 
-static UINT device_send_stream_list_request_pdu(CameraDeviceServerContext* context,
-                                                const CAM_STREAM_LIST_REQUEST* streamListRequest)
+static UINT device_send_stream_list_request_pdu(
+    CameraDeviceServerContext* context,
+    WINPR_ATTR_UNUSED const CAM_STREAM_LIST_REQUEST* streamListRequest)
 {
 	WINPR_ASSERT(context);
 
@@ -771,7 +779,7 @@ static UINT
 device_send_media_type_list_request_pdu(CameraDeviceServerContext* context,
                                         const CAM_MEDIA_TYPE_LIST_REQUEST* mediaTypeListRequest)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(mediaTypeListRequest);
@@ -789,7 +797,7 @@ static UINT device_send_current_media_type_request_pdu(
     CameraDeviceServerContext* context,
     const CAM_CURRENT_MEDIA_TYPE_REQUEST* currentMediaTypeRequest)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(currentMediaTypeRequest);
@@ -807,8 +815,7 @@ static UINT
 device_send_start_streams_request_pdu(CameraDeviceServerContext* context,
                                       const CAM_START_STREAMS_REQUEST* startStreamsRequest)
 {
-	wStream* s;
-	size_t i;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(startStreamsRequest);
@@ -818,28 +825,29 @@ device_send_start_streams_request_pdu(CameraDeviceServerContext* context,
 	if (!s)
 		return ERROR_NOT_ENOUGH_MEMORY;
 
-	for (i = 0; i < startStreamsRequest->N_Infos; ++i)
+	for (size_t i = 0; i < startStreamsRequest->N_Infos; ++i)
 	{
 		const CAM_START_STREAM_INFO* info = &startStreamsRequest->StartStreamsInfo[i];
 		const CAM_MEDIA_TYPE_DESCRIPTION* description = &info->MediaTypeDescription;
 
 		Stream_Write_UINT8(s, info->StreamIndex);
 
-		Stream_Write_UINT8(s, description->Format);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, description->Format));
 		Stream_Write_UINT32(s, description->Width);
 		Stream_Write_UINT32(s, description->Height);
 		Stream_Write_UINT32(s, description->FrameRateNumerator);
 		Stream_Write_UINT32(s, description->FrameRateDenominator);
 		Stream_Write_UINT32(s, description->PixelAspectRatioNumerator);
 		Stream_Write_UINT32(s, description->PixelAspectRatioDenominator);
-		Stream_Write_UINT8(s, description->Flags);
+		Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, description->Flags));
 	}
 
 	return device_server_packet_send(context, s);
 }
 
-static UINT device_send_stop_streams_request_pdu(CameraDeviceServerContext* context,
-                                                 const CAM_STOP_STREAMS_REQUEST* stopStreamsRequest)
+static UINT device_send_stop_streams_request_pdu(
+    CameraDeviceServerContext* context,
+    WINPR_ATTR_UNUSED const CAM_STOP_STREAMS_REQUEST* stopStreamsRequest)
 {
 	WINPR_ASSERT(context);
 
@@ -849,7 +857,7 @@ static UINT device_send_stop_streams_request_pdu(CameraDeviceServerContext* cont
 static UINT device_send_sample_request_pdu(CameraDeviceServerContext* context,
                                            const CAM_SAMPLE_REQUEST* sampleRequest)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(sampleRequest);
@@ -863,9 +871,9 @@ static UINT device_send_sample_request_pdu(CameraDeviceServerContext* context,
 	return device_server_packet_send(context, s);
 }
 
-static UINT
-device_send_property_list_request_pdu(CameraDeviceServerContext* context,
-                                      const CAM_PROPERTY_LIST_REQUEST* propertyListRequest)
+static UINT device_send_property_list_request_pdu(
+    CameraDeviceServerContext* context,
+    WINPR_ATTR_UNUSED const CAM_PROPERTY_LIST_REQUEST* propertyListRequest)
 {
 	WINPR_ASSERT(context);
 
@@ -876,7 +884,7 @@ static UINT
 device_send_property_value_request_pdu(CameraDeviceServerContext* context,
                                        const CAM_PROPERTY_VALUE_REQUEST* propertyValueRequest)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(propertyValueRequest);
@@ -885,7 +893,7 @@ device_send_property_value_request_pdu(CameraDeviceServerContext* context,
 	if (!s)
 		return ERROR_NOT_ENOUGH_MEMORY;
 
-	Stream_Write_UINT8(s, propertyValueRequest->PropertySet);
+	Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, propertyValueRequest->PropertySet));
 	Stream_Write_UINT8(s, propertyValueRequest->PropertyId);
 
 	return device_server_packet_send(context, s);
@@ -895,7 +903,7 @@ static UINT device_send_set_property_value_request_pdu(
     CameraDeviceServerContext* context,
     const CAM_SET_PROPERTY_VALUE_REQUEST* setPropertyValueRequest)
 {
-	wStream* s;
+	wStream* s = NULL;
 
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(setPropertyValueRequest);
@@ -905,10 +913,11 @@ static UINT device_send_set_property_value_request_pdu(
 	if (!s)
 		return ERROR_NOT_ENOUGH_MEMORY;
 
-	Stream_Write_UINT8(s, setPropertyValueRequest->PropertySet);
+	Stream_Write_UINT8(s, WINPR_ASSERTING_INT_CAST(uint8_t, setPropertyValueRequest->PropertySet));
 	Stream_Write_UINT8(s, setPropertyValueRequest->PropertyId);
 
-	Stream_Write_UINT8(s, setPropertyValueRequest->PropertyValue.Mode);
+	Stream_Write_UINT8(
+	    s, WINPR_ASSERTING_INT_CAST(uint8_t, setPropertyValueRequest->PropertyValue.Mode));
 	Stream_Write_INT32(s, setPropertyValueRequest->PropertyValue.Value);
 
 	return device_server_packet_send(context, s);
@@ -949,7 +958,10 @@ CameraDeviceServerContext* camera_device_server_context_new(HANDLE vcm)
 
 	return &device->context;
 fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	camera_device_server_context_free(&device->context);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
