@@ -51,6 +51,7 @@
 #include <freerdp/error.h>
 #include <freerdp/event.h>
 #include <freerdp/locale/keyboard.h>
+#include <freerdp/locale/locale.h>
 #include <freerdp/channels/channels.h>
 #include <freerdp/version.h>
 #include <freerdp/log.h>
@@ -87,7 +88,6 @@ static int freerdp_connect_begin(freerdp* instance)
 	rdpRdp* rdp = NULL;
 	BOOL status = TRUE;
 	rdpSettings* settings = NULL;
-	UINT32 KeyboardLayout = 0;
 
 	if (!instance)
 		return -1;
@@ -118,13 +118,22 @@ static int freerdp_connect_begin(freerdp* instance)
 	instance->ConnectionCallbackState = CLIENT_STATE_PRECONNECT_PASSED;
 
 	freerdp_settings_print_warnings(settings);
+	if (status)
+		status = freerdp_settings_enforce_monitor_exists(settings);
+
+	if (status)
+		status = freerdp_settings_check_client_after_preconnect(settings);
 
 	if (status)
 		status = rdp_set_backup_settings(rdp);
 	if (status)
 		status = utils_reload_channels(instance->context);
 
-	KeyboardLayout = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardLayout);
+	const UINT32 cp = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardCodePage);
+	int64_t KeyboardLayout = freerdp_get_keyboard_default_layout_for_locale(cp);
+	if (KeyboardLayout == 0)
+		KeyboardLayout = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardLayout);
+
 	switch (KeyboardLayout)
 	{
 		case KBD_JAPANESE:
@@ -149,7 +158,8 @@ static int freerdp_connect_begin(freerdp* instance)
 		WINPR_ASSERT(context);
 		freerdp_set_last_error_if_not(context, FREERDP_ERROR_PRE_CONNECT_FAILED);
 
-		WLog_Print(context->log, WLOG_ERROR, "freerdp_pre_connect failed");
+		WLog_Print(context->log, WLOG_ERROR, "freerdp_pre_connect failed: %s",
+		           rdp_client_connection_state_string(instance->ConnectionCallbackState));
 		return 0;
 	}
 
@@ -182,13 +192,8 @@ BOOL freerdp_connect(freerdp* instance)
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->settings);
 
-	if (rc < 0)
-		return FALSE;
-
-	if (rc == 0)
-		goto freerdp_connect_finally;
-
-	/* Pointers might have changed inbetween */
+	if (rc > 0)
+	/* Pointers might have changed in between */
 	{
 		rdp_update_internal* up = update_cast(rdp->update);
 
@@ -200,10 +205,7 @@ BOOL freerdp_connect(freerdp* instance)
 			if (up->pcap_rfx)
 				up->dump_rfx = TRUE;
 		}
-	}
 
-	if (rc > 0)
-	{
 		pointer_cache_register_callbacks(instance->context->update);
 		status = IFCALLRESULT(TRUE, instance->PostConnect, instance);
 		instance->ConnectionCallbackState = CLIENT_STATE_POSTCONNECT_PASSED;
@@ -297,6 +299,7 @@ freerdp_connect_finally:
 	return status;
 }
 
+#if !defined(WITHOUT_FREERDP_3x_DEPRECATED)
 BOOL freerdp_abort_connect(freerdp* instance)
 {
 	if (!instance)
@@ -304,6 +307,7 @@ BOOL freerdp_abort_connect(freerdp* instance)
 
 	return freerdp_abort_connect_context(instance->context);
 }
+#endif
 
 BOOL freerdp_abort_connect_context(rdpContext* context)
 {
@@ -312,11 +316,22 @@ BOOL freerdp_abort_connect_context(rdpContext* context)
 
 	freerdp_set_last_error_if_not(context, FREERDP_ERROR_CONNECT_CANCELLED);
 
+	/* Try to send a [MS-RDPBCGR] 1.3.1.4.1 User-Initiated on Client PDU, we don't care about
+	 * success */
+	if (context->rdp && context->rdp->mcs)
+	{
+		if (!context->ServerMode)
+		{
+			(void)mcs_send_disconnect_provider_ultimatum(context->rdp->mcs,
+			                                             Disconnect_Ultimatum_user_requested);
+		}
+	}
 	return utils_abort_connect(context->rdp);
 }
 
 #if defined(WITH_FREERDP_DEPRECATED)
-BOOL freerdp_get_fds(freerdp* instance, void** rfds, int* rcount, void** wfds, int* wcount)
+BOOL freerdp_get_fds(freerdp* instance, void** rfds, int* rcount, WINPR_ATTR_UNUSED void** wfds,
+                     WINPR_ATTR_UNUSED int* wcount)
 {
 	rdpRdp* rdp = NULL;
 
@@ -386,7 +401,11 @@ DWORD freerdp_get_event_handles(rdpContext* context, HANDLE* events, DWORD count
 	else
 		return 0;
 
-	return nCount;
+	const SSIZE_T rc = freerdp_client_channel_get_registered_event_handles(
+	    context->channels, &events[nCount], count - nCount);
+	if (rc < 0)
+		return 0;
+	return nCount + (DWORD)rc;
 }
 
 /* Resend mouse cursor position to prevent session lock in prevent-session-lock mode */
@@ -402,11 +421,12 @@ static BOOL freerdp_prevent_session_lock(rdpContext* context)
 	if (FakeMouseMotionInterval && in->lastInputTimestamp)
 	{
 		const time_t now = time(NULL);
-		if (now - in->lastInputTimestamp > FakeMouseMotionInterval)
+		if (WINPR_ASSERTING_INT_CAST(size_t, now) - in->lastInputTimestamp >
+		    FakeMouseMotionInterval)
 		{
 			WLog_Print(context->log, WLOG_DEBUG,
-			           "fake mouse move: x=%d y=%d lastInputTimestamp=%d "
-			           "FakeMouseMotionInterval=%d",
+			           "fake mouse move: x=%d y=%d lastInputTimestamp=%" PRIu64 " "
+			           "FakeMouseMotionInterval=%" PRIu32,
 			           in->lastX, in->lastY, in->lastInputTimestamp, FakeMouseMotionInterval);
 
 			BOOL status = freerdp_input_send_mouse_event(context->input, PTR_FLAGS_MOVE, in->lastX,
@@ -493,6 +513,8 @@ wMessageQueue* freerdp_get_message_queue(freerdp* instance, DWORD id)
 			queue = input->queue;
 		}
 		break;
+		default:
+			break;
 	}
 
 	return queue;
@@ -528,6 +550,8 @@ int freerdp_message_queue_process_message(freerdp* instance, DWORD id, wMessage*
 		case FREERDP_INPUT_MESSAGE_QUEUE:
 			status = input_message_queue_process_message(context->input, message);
 			break;
+		default:
+			break;
 	}
 
 	return status;
@@ -552,6 +576,8 @@ int freerdp_message_queue_process_pending_messages(freerdp* instance, DWORD id)
 		case FREERDP_INPUT_MESSAGE_QUEUE:
 			status = input_message_queue_process_pending_messages(context->input);
 			break;
+		default:
+			break;
 	}
 
 	return status;
@@ -612,11 +638,13 @@ BOOL freerdp_disconnect(freerdp* instance)
 	return rc;
 }
 
+#if !defined(WITHOUT_FREERDP_3x_DEPRECATED)
 BOOL freerdp_disconnect_before_reconnect(freerdp* instance)
 {
 	WINPR_ASSERT(instance);
 	return freerdp_disconnect_before_reconnect_context(instance->context);
 }
+#endif
 
 BOOL freerdp_disconnect_before_reconnect_context(rdpContext* context)
 {
@@ -645,6 +673,7 @@ BOOL freerdp_reconnect(freerdp* instance)
 	return rdp_client_reconnect(rdp);
 }
 
+#if !defined(WITHOUT_FREERDP_3x_DEPRECATED)
 BOOL freerdp_shall_disconnect(freerdp* instance)
 {
 	if (!instance)
@@ -652,8 +681,9 @@ BOOL freerdp_shall_disconnect(freerdp* instance)
 
 	return freerdp_shall_disconnect_context(instance->context);
 }
+#endif
 
-BOOL freerdp_shall_disconnect_context(rdpContext* context)
+BOOL freerdp_shall_disconnect_context(const rdpContext* context)
 {
 	if (!context)
 		return FALSE;
@@ -713,12 +743,15 @@ const char* freerdp_get_version_string(void)
 
 const char* freerdp_get_build_config(void)
 {
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_OVERLENGTH_STRINGS
 	static const char build_config[] =
 	    "Build configuration: " FREERDP_BUILD_CONFIG "\n"
 	    "Build type:          " FREERDP_BUILD_TYPE "\n"
 	    "CFLAGS:              " FREERDP_CFLAGS "\n"
 	    "Compiler:            " FREERDP_COMPILER_ID ", " FREERDP_COMPILER_VERSION "\n"
 	    "Target architecture: " FREERDP_TARGET_ARCH "\n";
+	WINPR_PRAGMA_DIAG_POP
 	return build_config;
 }
 
@@ -786,7 +819,6 @@ BOOL freerdp_context_new_ex(freerdp* instance, rdpSettings* settings)
 	if (!rdp)
 		goto fail;
 
-	rdp_log_build_warnings(rdp);
 	context->rdp = rdp;
 	context->pubSub = rdp->pubSub;
 
@@ -887,7 +919,7 @@ void freerdp_context_free(freerdp* instance)
 	ctx->metrics = NULL;
 
 	if (ctx->channelErrorEvent)
-		CloseHandle(ctx->channelErrorEvent);
+		(void)CloseHandle(ctx->channelErrorEvent);
 	ctx->channelErrorEvent = NULL;
 
 	free(ctx->errorDescription);
@@ -896,7 +928,7 @@ void freerdp_context_free(freerdp* instance)
 	freerdp_channels_free(ctx->channels);
 	ctx->channels = NULL;
 
-	codecs_free(ctx->codecs);
+	freerdp_client_codecs_free(ctx->codecs);
 	ctx->codecs = NULL;
 
 	stream_dump_free(ctx->dump);
@@ -1042,12 +1074,13 @@ void freerdp_set_last_error_ex(rdpContext* context, UINT32 lastError, const char
                                const char* file, int line)
 {
 	WINPR_ASSERT(context);
+	WINPR_ASSERT(line >= 0);
 
 	if (lastError)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_ERROR))
 		{
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, line, file, fkt,
+			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, (size_t)line, file, fkt,
 			                  "%s [0x%08" PRIX32 "]", freerdp_get_last_error_name(lastError),
 			                  lastError);
 		}
@@ -1056,14 +1089,14 @@ void freerdp_set_last_error_ex(rdpContext* context, UINT32 lastError, const char
 	if (lastError == FREERDP_ERROR_SUCCESS)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_DEBUG))
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_DEBUG, line, file, fkt,
+			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_DEBUG, (size_t)line, file, fkt,
 			                  "resetting error state");
 	}
 	else if (context->LastError != FREERDP_ERROR_SUCCESS)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_ERROR))
 		{
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, line, file, fkt,
+			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, (size_t)line, file, fkt,
 			                  "TODO: Trying to set error code %s, but %s already set!",
 			                  freerdp_get_last_error_name(lastError),
 			                  freerdp_get_last_error_name(context->LastError));
@@ -1075,7 +1108,7 @@ void freerdp_set_last_error_ex(rdpContext* context, UINT32 lastError, const char
 const char* freerdp_get_logon_error_info_type_ex(UINT32 type, char* buffer, size_t size)
 {
 	const char* str = freerdp_get_logon_error_info_type(type);
-	_snprintf(buffer, size, "%s(0x%04" PRIx32 ")", str, type);
+	(void)_snprintf(buffer, size, "%s(0x%04" PRIx32 ")", str, type);
 	return buffer;
 }
 
@@ -1130,7 +1163,7 @@ const char* freerdp_get_logon_error_info_data(UINT32 data)
 const char* freerdp_get_logon_error_info_data_ex(UINT32 data, char* buffer, size_t size)
 {
 	const char* str = freerdp_get_logon_error_info_data(data);
-	_snprintf(buffer, size, "%s(0x%04" PRIx32 ")", str, data);
+	(void)_snprintf(buffer, size, "%s(0x%04" PRIx32 ")", str, data);
 	return buffer;
 }
 
@@ -1165,7 +1198,8 @@ ULONG freerdp_get_transport_sent(rdpContext* context, BOOL resetCount)
 {
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->rdp);
-	return transport_get_bytes_sent(context->rdp->transport, resetCount);
+	UINT64 rc = transport_get_bytes_sent(context->rdp->transport, resetCount);
+	return WINPR_CXX_COMPAT_CAST(ULONG, MIN(rc, UINT32_MAX));
 }
 
 BOOL freerdp_nla_impersonate(rdpContext* context)
@@ -1204,15 +1238,39 @@ BOOL freerdp_nla_revert_to_self(rdpContext* context)
 
 UINT32 freerdp_get_nla_sspi_error(rdpContext* context)
 {
-	rdpNla* nla = NULL;
-
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->rdp);
 	WINPR_ASSERT(context->rdp->transport);
 
-	nla = transport_get_nla(context->rdp->transport);
+	rdpNla* nla = transport_get_nla(context->rdp->transport);
 
-	return nla_get_sspi_error(nla);
+	return (UINT32)nla_get_sspi_error(nla);
+}
+
+BOOL freerdp_nla_encrypt(rdpContext* context, const SecBuffer* inBuffer, SecBuffer* outBuffer)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+
+	rdpNla* nla = context->rdp->nla;
+	return nla_encrypt(nla, inBuffer, outBuffer);
+}
+
+BOOL freerdp_nla_decrypt(rdpContext* context, const SecBuffer* inBuffer, SecBuffer* outBuffer)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+
+	rdpNla* nla = context->rdp->nla;
+	return nla_decrypt(nla, inBuffer, outBuffer);
+}
+
+SECURITY_STATUS freerdp_nla_QueryContextAttributes(rdpContext* context, DWORD ulAttr, PVOID pBuffer)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+
+	return nla_QueryContextAttributes(context->rdp->nla, ulAttr, pBuffer);
 }
 
 HANDLE getChannelErrorEventHandle(rdpContext* context)
@@ -1257,21 +1315,21 @@ void clearChannelError(rdpContext* context)
 	WINPR_ASSERT(context);
 	context->channelErrorNum = 0;
 	memset(context->errorDescription, 0, 500);
-	ResetEvent(context->channelErrorEvent);
+	(void)ResetEvent(context->channelErrorEvent);
 }
 
 WINPR_ATTR_FORMAT_ARG(3, 4)
 void setChannelError(rdpContext* context, UINT errorNum, WINPR_FORMAT_ARG const char* format, ...)
 {
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, format);
 
 	WINPR_ASSERT(context);
 
 	context->channelErrorNum = errorNum;
-	vsnprintf(context->errorDescription, 499, format, ap);
+	(void)vsnprintf(context->errorDescription, 499, format, ap);
 	va_end(ap);
-	SetEvent(context->channelErrorEvent);
+	(void)SetEvent(context->channelErrorEvent);
 }
 
 const char* freerdp_nego_get_routing_token(rdpContext* context, DWORD* length)
@@ -1419,3 +1477,30 @@ BOOL freerdp_is_valid_mcs_create_response(const BYTE* data, size_t size)
 	test_mcs_free(mcs);
 	return result;
 }
+
+BOOL freerdp_persist_credentials(rdpContext* context)
+{
+	if (!context)
+		return FALSE;
+	WINPR_ASSERT(context->rdp);
+	return utils_persist_credentials(context->rdp->originalSettings, context->rdp->settings);
+}
+
+const char* freerdp_disconnect_reason_string(int reason)
+{
+	switch (reason)
+	{
+		case Disconnect_Ultimatum_domain_disconnected:
+			return "rn-domain-disconnected";
+		case Disconnect_Ultimatum_provider_initiated:
+			return "rn-provider-initiated";
+		case Disconnect_Ultimatum_token_purged:
+			return "rn-token-purged";
+		case Disconnect_Ultimatum_user_requested:
+			return "rn-user-requested";
+		case Disconnect_Ultimatum_channel_purged:
+			return "rn-channel-purged";
+		default:
+			return "rn-unknown";
+	}
+}
